Exception: Darkroom::AssetNotFoundError
- Inherits:
-
AssetError
- Object
- StandardError
- AssetError
- Darkroom::AssetNotFoundError
- Defined in:
- lib/darkroom/errors/asset_not_found_error.rb
Overview
Error class used when an asset requested explicitly or specified as a dependency of another doesn’t exist.
Instance Attribute Summary
Attributes inherited from AssetError
#detail, #source_line_num, #source_path
Instance Method Summary collapse
-
#initialize(path, source_path = nil, source_line_num = nil) ⇒ AssetNotFoundError
constructor
Public: Create a new instance.
Constructor Details
#initialize(path, source_path = nil, source_line_num = nil) ⇒ AssetNotFoundError
Public: Create a new instance.
path - String path of asset that doesn’t exist. source_path - String path of the asset that contains the error. source_line_num - Integer line number in the asset file where the error is located.
14 15 16 |
# File 'lib/darkroom/errors/asset_not_found_error.rb', line 14 def initialize(path, source_path = nil, source_line_num = nil) super('Asset not found', path, source_path, source_line_num) end |